﻿using System.Net;
using System.Net.Http;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json;
using Polly;
using Polly.Caching;
using Polly.Registry;

namespace CachingValuesInsideHttpResponsesInPolly.Controllers
{
    [Produces("application/json")]
    [Route("api/[controller]")]
    public class CatalogController : Controller
    {
        private readonly IHttpClientFactory _httpClientFactory;
        private readonly IPolicyRegistry<string> _policyRegistry;

        public CatalogController(IHttpClientFactory httpClientFactory, IPolicyRegistry<string> policyRegistry)
        {
            _httpClientFactory = httpClientFactory;
            _policyRegistry = policyRegistry;
        }

        [HttpGet("{id}")]
        public async Task<IActionResult> Get(int id)
        {
            string requestEndpoint = $"inventory/{id}";
            var httpClient = _httpClientFactory.CreateClient("RemoteServer");

            var cachePolicy = _policyRegistry.Get<CachePolicy<int>>("CachingPolicy");
            Context policyExecutionContext = new Context($"GetInventoryById-{id}");

            int itemsInStock =
                await cachePolicy.ExecuteAsync(context => GetItemsInStockCount(), policyExecutionContext);

            async Task<int> GetItemsInStockCount()
            {
                HttpResponseMessage response = await httpClient.GetAsync(requestEndpoint);

                if (response.IsSuccessStatusCode)
                {
                    int itemsInStockCount = await response.Content.ReadAsAsync<int>();
                    return itemsInStockCount;
                }

                return 0;
            }

            return Ok(itemsInStock);
        }
    }
}
